package com.netease.nis.ocr_plugin_flutter.ocr_plugin_flutter

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.netease.nis.basesdk.Logger
import com.netease.nis.ocr.OcrCropListener
import com.netease.nis.ocr.OcrScanView
import com.netease.nis.ocr.OcrScanner
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel.Result

/**
 * @author liuxiaoshuai
 * @date 2023/2/23
 * @desc
 * @email liulingfeng@mistong.com
 */
class OcrHelper {
    private var ocrScanner = OcrScanner.getInstance()
    var events: EventChannel.EventSink? = null
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    private var cameraPreview: OcrScanView? = null
    fun init(
        context: Context,
        cameraPreview: OcrScanView?,
        businessId: String,
        timeout: Int,
        isDebug: Boolean,
    ) {
        if (cameraPreview == null) {
            Log.e(OcrScanner.TAG, "预览view为空")
            return
        }
        this.cameraPreview = cameraPreview
        ocrScanner.init(context, cameraPreview, businessId)
        ocrScanner.setDebug(isDebug)
        ocrScanner.setScanType(OcrScanner.SCAN_TYPE_AVATAR)
        ocrScanner.setTimeOut(timeout.toLong() * 1000)
    }

    fun setScanType(scanType: Int) {
        if (scanType >= OcrScanner.SCAN_TYPE_NATIONAL_EMBLEM && scanType <= OcrScanner.SCAN_TYPE_AVATAR) {
            ocrScanner.setScanType(scanType)
        }
    }

    fun startScanner() {
        val map = HashMap<String, Any?>()
        ocrScanner.setCropListener(object : OcrCropListener {
            override fun onSuccess(picturePath: String?) {
                uiThreadHandler.post {
                    map["type"] = "success"
                    map["picturePath"] = picturePath
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(OcrScanner.TAG, "Reply already submitted")
                    }
                }
                Logger.d(OcrScanner.TAG, "保存的图片路径为$picturePath")
            }

            override fun onError(code: Int, msg: String?) {
                uiThreadHandler.post {
                    map["type"] = "exception"
                    map["code"] = code
                    map["message"] = msg
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(OcrScanner.TAG, "Reply already submitted")
                    }
                }
                Logger.e(OcrScanner.TAG, "ocr扫描出错，原因:$msg")
            }

            override fun onOverTime() {
                uiThreadHandler.post {
                    map["type"] = "overTime"
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(OcrScanner.TAG, "Reply already submitted")
                    }
                }
                Logger.d(OcrScanner.TAG, "ocr扫描检测超时")
            }

        })
        cameraPreview?.postDelayed({
            ocrScanner.start()
        }, 500)
    }

    fun stopScanner() {
        ocrScanner.stop()
    }

    fun destroy() {
        ocrScanner.destroy()
    }
}