package com.netease.nis.ocr_plugin_flutter.ocr_plugin_flutter

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** OcrPluginFlutterPlugin */
class OcrPluginFlutterPlugin : FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var context: Context
    private var ocrHelper: OcrHelper? = null
    private lateinit var nativeViewFactory: NativeViewFactory
    private val EVENT_CHANNEL = "yd_ocr_flutter_event_channel"

    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        context = binding.applicationContext
        channel = MethodChannel(binding.binaryMessenger, "ocr_plugin_flutter")
        channel.setMethodCallHandler(this)
        nativeViewFactory = NativeViewFactory()
        binding.platformViewRegistry.registerViewFactory("platform-view-ocr", nativeViewFactory)
        ocrHelper = OcrHelper()
        registerEventHandle(binding.binaryMessenger)
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "init" -> {
                val businessId = call.argument<String>("businessID")
                val timeout = call.argument<Int>("timeout") ?: 120 // 超时时间
                val isDebug = call.argument<Boolean>("is_debug") ?: false // 是否debug模式

                if (businessId == null) {
                    Log.e("AliveFlutterPlugin", "业务id不能为空")
                    return
                }
                val ocrView = nativeViewFactory.ocrView
                ocrHelper?.init(
                    context,
                    ocrView.getCameraView(),
                    businessId,
                    timeout,
                    isDebug
                )
            }
            "setScanType" -> {
                val scanType = call.argument<Int>("scanType") ?: 1
                ocrHelper?.setScanType(scanType)
            }
            "startScanner" -> {
                ocrHelper?.startScanner()
            }
            "stopScanner" -> {
                ocrHelper?.stopScanner()
            }
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "destroy" -> {
                ocrHelper?.destroy()
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun registerEventHandle(message: BinaryMessenger) {
        EventChannel(message, EVENT_CHANNEL).setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
                ocrHelper?.events = events
            }

            override fun onCancel(arguments: Any?) {

            }
        })
    }
}
