package com.netease.nis.ocr_plugin_flutter.ocr_plugin_flutter

import android.content.Context
import android.util.DisplayMetrics
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import com.netease.nis.ocr.OcrScanView
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView

/**
 * @author liuxiaoshuai
 * @date 2023/2/23
 * @desc
 * @email liulingfeng@mistong.com
 */
class OcrView(context: Context, id: Int, creationParams: Map<String?, Any?>?) : PlatformView,
    MethodChannel.MethodCallHandler {

    private var cameraPreview: FrameLayout
    private var params: Map<String?, Any?>? = creationParams

    init {
        cameraPreview = LayoutInflater.from(context)
            .inflate(getLayoutId(context), null) as FrameLayout
    }

    fun getCameraView(): OcrScanView {
        val cameraView: OcrScanView = cameraPreview.findViewById(R.id.ocr_view)
        params?.let {
            if (it.containsKey("width") && it.containsKey("height")) {
                val width = dpTpPx(cameraView, (it["width"] as Int).toFloat())
                val height = dpTpPx(cameraView, (it["height"] as Int).toFloat())
                val layoutParams = FrameLayout.LayoutParams(width.toInt(), height.toInt())
                layoutParams.gravity = Gravity.CENTER
                cameraView.layoutParams = layoutParams
            }
        }
        return cameraView
    }

    override fun getView(): View {
        return cameraPreview
    }

    override fun dispose() {
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
    }

    private fun getLayoutId(context: Context): Int {
        return context.resources.getIdentifier("ocr_view_layout", "layout", context.packageName)
    }

    private fun dpTpPx(view: View, value: Float): Double {
        val dm: DisplayMetrics = view.resources.displayMetrics
        return (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, value, dm) + 0.5)
    }
}