import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:ocr_plugin_flutter/ocr_plugin_flutter.dart';
import 'dart:ui';

class OcrScan extends StatefulWidget {
  final String scanType;
  const OcrScan({Key? key, this.scanType = "front"}) : super(key: key);

  @override
  State<OcrScan> createState() => _MyOcrScanState();
}

class _MyOcrScanState extends State<OcrScan> {
  Image _imageFront = Image.asset("images/detectBorder.png");


  final _ocrPluginFlutterPlugin = OcrPluginFlutter();
  var eventChannel = const EventChannel("yd_ocr_flutter_event_channel");
   Size sizeWindow = Size(0, 0);

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      init();
    });
    eventChannel.receiveBroadcastStream().listen(_onData, onError: _onError);
  }

  void init() {
    _ocrPluginFlutterPlugin.init("be66035041fe4704a1207b6134c23254", 120, true);
    if ("front" == widget.scanType) {
      _ocrPluginFlutterPlugin.setScanType(1);
    } else {
      _ocrPluginFlutterPlugin.setScanType(0);
    }

    _ocrPluginFlutterPlugin.startScanner();
  }

  void _onData(response) {
    if (response is Map) {
      if (mounted) {
        setState(() {
          var type = response["type"];
          if (type == "success") {
            if (kDebugMode) {
              print("检测成功${response["picturePath"]}");
              _ocrPluginFlutterPlugin.stopScanner();
            }
            Navigator.of(context).pop(response["picturePath"]);
          } else if (type == "exception") {
            if (kDebugMode) {
              print("检测异常${response["message"]}");
            }
          } else if (type == "overTime") {
            if (kDebugMode) {
              print("检测超时");
            }
          }
        });
      }
    }
  }

  _onError(Object error) {}

  @override
  Widget build(BuildContext context) {
    sizeWindow = MediaQuery.of(context).size;;
    // screenWitdh = size.width;
    // screenHeight = size.height;
    // print('width is $width; height is $height');

    return WillPopScope(
        onWillPop: () async {
          // Android放在这里提前处理，否则有残影
          if (defaultTargetPlatform == TargetPlatform.android) {
            _ocrPluginFlutterPlugin.stopScanner();
            //android需要释放资源，否则会引发内存泄露
            _ocrPluginFlutterPlugin.destroy();
          }
          return true;
        },
        child: MaterialApp(
          home: Scaffold(
            appBar: AppBar(
              title: const Text('身份证采集'),
            ),
            body: Container (
              child: Stack(
                children: [
                  showOcrImageWidget(),
                  showOcrDetectWidget(),
                  Center(
                    child: Container(
                      width: 300,
                      margin: const EdgeInsets.fromLTRB(20, 120, 20, 0),
                      child: const Text("保持光线明亮",
                          style: TextStyle(color: Colors.white, fontSize: 18),
                          textAlign: TextAlign.left),
                    ),
                  ),
                  Center(
                    child: Container(
                      width: 300,
                      margin: const EdgeInsets.fromLTRB(20, 120, 20, 0),
                      child: const Text("保持光线明亮",
                          style: TextStyle(color: Colors.white, fontSize: 18),
                          textAlign: TextAlign.left),
                    ),
                  ),
                  Center(
                    child: Container(
                      width: 300,
                      margin: const EdgeInsets.fromLTRB(20, 180, 20, 0),
                      child: const Text("将身份证人像面放入采集框中",
                          style: TextStyle(color: Colors.white, fontSize: 18),
                          textAlign: TextAlign.left),
                    ),
                  ),
                  Center(
                    child: Container(
                      width: 300,
                      margin: const EdgeInsets.fromLTRB(20, 240, 20, 0),
                      child: const Text("文字不要有反光、遮挡",
                          style: TextStyle(color: Colors.white, fontSize: 18),
                          textAlign: TextAlign.left),
                    ),
                  )
                ],
              ),
            ),
          ),
        ));
  }

  Widget showOcrDetectWidget() {
    // ocr 身份证识别区域
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      return Container(
          padding: EdgeInsets.only(top: 100,left: 10,right: 10),
          // width: GlobalUtils.,
          // height: 200,
          width: sizeWindow.width,
          height: 400,
          child: _imageFront);
    }
    return Text('$defaultTargetPlatform is not yet supported by this plugin');
  }

  Widget showOcrImageWidget() {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return Container(
        alignment: Alignment.center,
        child: PlatformViewLink(
          viewType: "platform-view-ocr",
          surfaceFactory: (context, controller) {
            return AndroidViewSurface(
              controller: controller as AndroidViewController,
              gestureRecognizers: const <
                  Factory<OneSequenceGestureRecognizer>>{},
              hitTestBehavior: PlatformViewHitTestBehavior.opaque,
            );
          },
          onCreatePlatformView: (params) {
            return PlatformViewsService.initExpensiveAndroidView(
              id: params.id,
              viewType: "platform-view-ocr",
              layoutDirection: TextDirection.ltr,
              creationParams: {},
              creationParamsCodec: const StandardMessageCodec(),
              onFocus: () {
                params.onFocusChanged(true);
              },
            )
              ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
              ..create();
          },
        ),
      );
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      return new Center(
        child: Container(
          child: UiKitView(
            viewType: 'com.flutter.ocr.imageview',
            // 身份证扫描框的坐标
            creationParams: {
              "width": sizeWindow.width,
              "height": 400,
              "x":5,
              "y":50,
              // "radius": "100",
            },
            //参数的编码方式
            creationParamsCodec: const StandardMessageCodec(),
          ),
        ),
      );
    }
    return Text('$defaultTargetPlatform is not yet supported by this plugin');
  }

  @override
  void dispose() {
    super.dispose();
  }
}
