#import "OcrPluginFlutterPlugin.h"
#import <NTESOcrSDK/NTESOcrSDK.h>
#import "OcrPlatformViewFactory.h"
#if __has_include(<ocr_plugin_flutter/ocr_plugin_flutter-Swift.h>)
#import <ocr_plugin_flutter/ocr_plugin_flutter-Swift.h>
#else
// Support project import fallback if the generated compatibility header
// is not copied when this plugin is created as a library.
// https://forums.swift.org/t/swift-static-libraries-dont-copy-generated-objective-c-header/19816
#import "ocr_plugin_flutter-Swift.h"
#endif

@interface OcrPluginFlutterPlugin()

@property (nonatomic, strong) NTESOcrSdkManager *manager;
@property (nonatomic, copy) NSDictionary *params;

@property (nonatomic, copy) NSString *businessID;
@property (nonatomic, assign) int timeout;

@property (nonatomic, copy) NSString *scanType;

@end

@implementation OcrPluginFlutterPlugin

+ (OcrPluginFlutterPlugin *)sharedInstance {
    static dispatch_once_t onceToken = 0;
    static OcrPluginFlutterPlugin *sharedObject = nil;
    dispatch_once(&onceToken, ^{
        sharedObject = [[OcrPluginFlutterPlugin alloc] init];
    });
    
    return sharedObject;
}

NSObject<FlutterPluginRegistrar>* _ocr_registrar;

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {

    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"ocr_plugin_flutter" binaryMessenger:[registrar messenger]];
    _ocr_registrar = registrar;
    OcrPluginFlutterPlugin* instance = [[OcrPluginFlutterPlugin alloc] init];
    instance.channel = channel;
    [registrar addMethodCallDelegate:instance channel:channel];

    FlutterViewController *view = (FlutterViewController *)[UIApplication sharedApplication].delegate.window.rootViewController;
    FlutterEventChannel* eventChannel= [FlutterEventChannel eventChannelWithName:@"yd_ocr_flutter_event_channel" binaryMessenger:view.binaryMessenger];
    [eventChannel setStreamHandler:[OcrPluginFlutterPlugin sharedInstance]];
    
    [registrar registerViewFactory:[OcrPlatformViewFactory sharedInstance] withId:@"com.flutter.ocr.imageview"];
}


- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments
                                      eventSink:(FlutterEventSink)events {
      if (events) {
          [OcrPluginFlutterPlugin sharedInstance].eventSink = events;
      }

    return nil;
}

- (FlutterError * _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    return nil;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSString *methodName = call.method;
    if ([methodName isEqualToString:@"init"]) {
        [self init:call result:result];
    } else if ([methodName isEqualToString:@"startScanner"]) {
        [self startScanner:call result:result];
    } else if ([methodName isEqualToString:@"init"]) {
        [self init:call result:result];
    }  else if ([methodName isEqualToString:@"setScanType"])  {
        [self setScanType:call result:result];
    }  else if ([methodName isEqualToString:@"stopScanner"])  {
        [self stopScanner];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)init:(FlutterMethodCall*)call result:(FlutterResult)resultDict {
    NSDictionary *arguments = call.arguments;
    NSLog(@"%@====",arguments);
    self.timeout = 0;
    if ([arguments isKindOfClass:[NSDictionary class]]) {
        self.businessID = [arguments objectForKey:@"businessID"];
        int time = [[arguments objectForKey:@"timeout"] intValue];
        if (time <= 0) {
            self.timeout = 30;
        } else {
            self.timeout = time;
        }
    }
}

- (void)stopScanner {
    [self.manager stopDetect];
}

- (void)setScanType:(FlutterMethodCall*)call result:(FlutterResult)resultDict {
    NSDictionary *arguments = call.arguments;
    if ([arguments isKindOfClass:[NSDictionary class]]) {
        self.scanType = [arguments objectForKey:@"scanType"];
    }
}

- (void)startScanner:(FlutterMethodCall*)call result:(FlutterResult)resultDict {
    OcrPlatformViewObject *object = [OcrPlatformViewFactory sharedInstance].myPlatformViewObject;
    UIImageView *imageView = object.imageView;
    
    NSLog(@"%d====",object.x);
    NSLog(@"%d====",object.y);
    NSLog(@"%d====",object.width);
    NSLog(@"%d====",object.height);
    self.manager = [[NTESOcrSdkManager alloc] initWithImageView:imageView scanRect:CGRectMake(object.x, object.y, object.width, object.height)];
    [self.manager setTimeoutInterval:self.timeout];
//    WeakSelf(self);
    NTESCardReverseType type = NTESCardReverseTypeFront;
    if ([self.scanType intValue] == 1) {
        type = NTESCardReverseTypeFront;
    } else if ([self.scanType intValue] == 0) {
        type = NTESCardReverseTypeBack;
    } else {
        type = NTESCardReverseTypeFront;
    }
    
    __weak __typeof__(self) weakSelf = self;
    [self.manager startDetectWithBusinessID:self.businessID cardReverseType:type completionHandler:^(NTESOcrStatus status, NSDictionary * _Nullable params) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [weakSelf showToastWithLiveDetectStatus:status params:params];
        });
    }];
}

- (void)showToastWithLiveDetectStatus:(NTESOcrStatus)status params:(NSDictionary * _Nullable) params {
    NSDictionary *callbackDict;
    if (status == NTESOcrOperationTimeout) {
        [self.manager stopDetect];
        callbackDict = @{
            @"type":@"overTime"
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    } else if (status == NTESOcrCheckPass) {
        NSString *imageFilePath = [params objectForKey:@"imageFilePath"];
        callbackDict = @{
            @"type":@"success",
            @"picturePath":imageFilePath?:@""
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    } else if (status == NTESOcrCheckNotPass) {
        callbackDict = @{
            @"type":@"exception",
            @"message":@"检测不通过"
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    } else if (status == NTESOcrBusinessIdInvalid) {
        callbackDict = @{
            @"type":@"exception",
            @"message":@"业务ID不可用"
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    } else if (status == NTESOcrNonGateway) {
        callbackDict = @{
            @"type":@"exception",
            @"message":@"网络不可用"
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    } else if (status == NTESOcrCameraNotAvailable) {
        callbackDict = @{
            @"type":@"exception",
            @"message":@"APP未获得手机权限"
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    } else if (status == NTESOcrGetConfTimeout) {
        callbackDict = @{
            @"type":@"exception",
            @"message":@"获取配置信息超时"
        };
        if ([OcrPluginFlutterPlugin sharedInstance].eventSink) {
            [OcrPluginFlutterPlugin sharedInstance].eventSink(callbackDict);
        }
    }
    dispatch_async(dispatch_get_main_queue(), ^(){

    });
}

@end
